#!/usr/bin/env python
# 
# Written by John Halloran <halloj3@uw.edu>
# 
# Copyright 2014, John Halloran, All rights reserved. 
# 
# Permission to use, copy, modify, and distribute this
# software and its documentation for any non-commercial purpose
# and without fee is hereby granted, provided that the above copyright
# notice appears in all copies.  The University of Washington,
# Seattle, and John Halloran make no representations about
# the suitability of this software for any purpose.  It is provided
# "as is" without express or implied warranty.


"""Support for absolute ranking plots"""

import argparse
import csv
import itertools

import matplotlib
matplotlib.use('Agg')
import pylab

from plotFDR import calcQ

def load_psms(filename):
    """ Load psms from an ident file.
    Pre:
        Ident file filename is tab-delimited with fields(order unspecific):
        Sid - spectrum scan id(from dataset)
        Kind - target or decoy, denoted as t or d, respectively
        Score

        There may be more than a single target or decoy per sid, but all
        targets and decoys of the same sid must be listed successively in the
        ident file.  Otherwise, groupby will give undesirable outputs.
    Post:
        Lists of tuples targets and decoys containing the top psm per sid, such
        that for element x in targets/decoys, x[0] = sid, x[1] = peptide string,
        x[2] =score
    """
    targets = []
    decoys = []
    f = open(filename)
    reader = csv.DictReader(f, delimiter = '\t', skipinitialspace = True)

    isTarget = lambda r: r["Kind"] == "t"
    isDecoy = lambda r: r["Kind"] == "d"
    score = lambda r: float(r["Score"])
    for sid, rows in itertools.groupby(reader, lambda r: int(r["Sid"])):
        records = list(rows)
        try:
            # take max target psm
            target = max(itertools.ifilter(isTarget, records), key = score)
            targets.append( ( int(sid), target["Peptide"],
                              float(target["Score"])))
            # take max decoy psm
            decoy = max(itertools.ifilter(isDecoy, records), key = score)
            decoys.append( ( int(sid), decoy["Peptide"],
                             float(decoy["Score"])))

        except ValueError:
            print 'Error processing line %d in file %s' % (reader.line_num, f.name)
            raise

    f.close()

    assert len(targets)==len(decoys)
    spectrumId = lambda r: r[0]
    targets.sort(key = spectrumId)
    decoys.sort(key = spectrumId)
    return targets, decoys

def load_benchmarks(args):
    """ Load all provided benchmark ident files and return a list of
        all target and decoy scores per benchmark for the intersection of
        their scored spectra
    """
    # load all input benchmarks and
    # find sid intersection between all methods
    benchmarks = []
    sids = set([])
    if args.drip:
        t,d = load_psms(args.drip)
        benchmarks.append((t,d,'DRIP'))
        if args.verbose:
            print "Loaded %d psms from %s." % (len(t), args.drip)
        # find sid intersection
        if sids:
            sids = sids & set([x[0] for x in t])
        else:
            sids = set([x[0] for x in t])
    if args.didea:
        t,d = load_psms(args.didea)
        if args.verbose:
            print "Loaded %d psms from %s." % (len(t), args.didea)
        benchmarks.append((t,d,'Didea'))
        # find sid intersection
        if sids:
            sids = sids & set([x[0] for x in t])
        else:
            sids = set([x[0] for x in t])
    if args.msgfdb:
        t,d = load_psms(args.msgfdb)
        if args.verbose:
            print "Loaded %d psms from %s." % (len(t), args.msgfdb)
        benchmarks.append((t,d,'MS-GFDB'))
        # find sid intersection
        if sids:
            sids = sids & set([x[0] for x in t])
        else:
            sids = set([x[0] for x in t])
    if args.sequest:
        t,d = load_psms(args.sequest)
        if args.verbose:
            print "Loaded %d psms from %s." % (len(t), args.sequest)
        benchmarks.append((t,d,'SEQUEST'))
        # find sid intersection
        if sids:
            sids = sids & set([x[0] for x in t])
        else:
            sids = set([x[0] for x in t])
    if args.omssa:
        t,d = load_psms(args.omssa)
        if args.verbose:
            print "Loaded %d psms from %s." % (len(t), args.omssa)
        benchmarks.append((t,d,'OMSSA'))
        # find sid intersection
        if sids:
            sids = sids & set([x[0] for x in t])
        else:
            sids = set([x[0] for x in t])

    if args.verbose:
        print "%d mutual scored spectra between all methods." % len(sids)

    # filter benchmark psms that don't lie in set sids
    filtered_benchmarks = []
    sidFilter = lambda r: r[0] in sids
    for t,d,method in benchmarks:
        targets = list(itertools.ifilter(sidFilter,t))
        decoys = list(itertools.ifilter(sidFilter,d))
        # only retain scores from here on
        filtered_benchmarks.append(([x[2] for x in targets],
                                    [x[2] for x in decoys],
                                    method))

    return filtered_benchmarks

def plot_performance(args):
    """ Plot q-value vs. num spectra identified at a specific q-value for all
        provided benchmark methods, saving the resulting figure to args.output.
    """
    pylab.clf()
    pylab.grid()
    pylab.xlabel('q-value')
    pylab.ylabel('Num spectra identified')
    pylab.spectral()
    pylab.gray()

    method_colors = ('k', 'b', 'g', 'r', 'y', 'm', 'c')

    plotted_methods = []
    ymax = 0
    for i, (target_scores,decoy_scores,method) in enumerate(load_benchmarks(args)):
        qs, num_spectra = calcQ(target_scores,decoy_scores)
        plotted_methods.append(method)
        # restrict to provided maximum q-value range
        x = []
        y = []
        for q,n in zip(qs,num_spectra):
            if q <= args.maxq:
                x.append(q)
                y.append(n)
            else:
                break

        pylab.plot(x,y, color = method_colors[i], linewidth = 2)
        ymax = max(ymax, y[-1])

    pylab.xlim(0.0,args.maxq)
    pylab.ylim(0.0, ymax)
    pylab.legend(plotted_methods, loc = 'best')
    pylab.savefig(args.output)
    if args.plot_title:
        pylab.suptitle(args.plot_title)
    

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Benchmark several MS/MS search algorithms.")
    help_drip = "DRIP ident file"
    parser.add_argument("--drip_ident", type = str, help = help_drip, dest = "drip", default = None)
    help_didea = "Didea ident file"
    parser.add_argument("--didea_ident", type = str, help = help_didea, dest = "didea", default = None)
    help_msgfdb = "MS-GFDB ident file"
    parser.add_argument("--msgfdb_ident", type = str, help = help_msgfdb, dest = "msgfdb", default = None)
    help_sequest = "SEQUEST ident file"
    parser.add_argument("--sequest_ident", type = str, help = help_sequest, dest = "sequest", default = None)
    help_omssa = "OMSSA ident file"
    parser.add_argument("--omssa_ident", type = str, help = help_omssa, dest = "omssa", default = None)
    help_output = "Plot output file"
    parser.add_argument("--output", type = str, help = help_output)
    parser.add_argument("--plot_title", type = str, help = "Figure title", default = None)
    parser.add_argument("--maxq", type = float, help = "Max q-value range to plot", default = 0.1)
    parser.add_argument("--verbose", action = "store_true", default = False)
    args = parser.parse_args()

    # check inputs
    if not args.output:
        print "Output file name required, exitting."
        exit(1)
    if not args.drip and not args.didea and not args.msgfdb and not args.sequest and not args.omssa:
        print "At least one identificaiton file must be supplied, exitting."
        exit(2)
    # make sure valid q-value range is input
    if args.maxq <= 0.0 or args.maxq > 1.0:
        print "Supplied q-value max must be greater than 0 and less than 1, exitting."
        exit(3)

    # plot methods of interest
    plot_performance(args)
